package classUtils.delegate;

import gui.ClosableJFrame;
import gui.run.RunButton;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Point;
import java.util.Vector;

/**
 * DelegateGui class To replace inheritance with
 * delegation: I dentify the super class. Make an
 * instance of the super class in the child. Using
 * semiautomatic static proxy delegation create a
 * proxy for the primordial super-class. Remove
 * the extends clause. Replace all references to
 * the parent with references to the proxy.
 */
public class DelegateGui {
    private JFrame jf = new ClosableJFrame("Disambiguate");
    private JLabel dupLabel = getDuplicateMethodsLabel();
    private JList duplicateJList = new JList();
    private JList doJList = new JList();
    private JScrollPane duplicateScrollPane = new JScrollPane();
    private JScrollPane selectedMethodsScrollPane = new JScrollPane();
    private JLabel doLabel = getDoLabel();
    private JPanel listPanel = getScrollPanels();
    private JPanel buttonPanel = buttonPanel();
    private DelegateSynthesizer ds = null;
    private ReflectUtil reflectUtil = new ReflectUtil(getDelegateSynthesizer());

    public DelegateGui() {
    }

    public void setVisible(boolean v) {
        jf.setVisible(v);
    }

    public void setInstanceList(Vector v) {
        DelegateSynthesizer ds = getDelegateSynthesizer();
        for (int i = 0; i < v.size(); i++)
            ds.add(v.elementAt(i));
        ds.process();
        System.out.println(ds.getClassString()
                + ds.getInterface());
        duplicateJList.setListData(ds.getDupMethodList());
        dupLabel.setText("Duplicate Methods: "
                +
                ds.getDupMethodList()
                .size());
        doLabel.setText("selected Method: " +
                ds.getDoMethodList()
                .size());
    }

    public void init() {
        buildScrollPane();
        duplicateJList.setVisible(true);
        initDoScrollPane();
        doJList.setVisible(true);
        initListPanel();
        addPanelsSetLayout();
        jf.setSize(650, 250);
    }

    private void addPanelsSetLayout() {
        Container c = jf.getContentPane();
        c.setLayout(new BorderLayout());
        c.add(getCounterPanel(), "North");
        c.add(listPanel, "Center");
        c.add(buttonPanel, "South");
    }

    private JPanel getScrollPanels() {
        JPanel jp = new JPanel();
        jp.setLayout(new GridLayout(1,0));
        jp.add(duplicateScrollPane);
        jp.add(selectedMethodsScrollPane);
        return jp;
    }

    private JPanel getCounterPanel() {
        JPanel jp = new JPanel();
        jp.add(dupLabel);
        jp.add(doLabel);
        jp.setLayout(new GridLayout(1, 0));
        return jp;
    }

    private JPanel buttonPanel() {
        JPanel jp = new JPanel();
        jp.setLayout(new FlowLayout());
        jp.add(new RunButton("OK") {
            public void run() {
                doOk();
            }
        });
        jp.add(new RunButton("add") {
            public void run() {
                doAdd();
            }
        });
        jp.add(new RunButton("Delete") {
            public void run() {
                doDelete();
            }
        });
        jp.add(new RunButton("topological sorting") {
            public void run() {
                doAdd();
            }
        });
        jp.add(new RunButton("exit") {
            public void run() {
                exit();
            }
        });
        return jp;
    }

    private JLabel getDoLabel() {
        JLabel l = new JLabel("Selected Method");
        l.setSize(new Dimension(140, 20));
        l.setVisible(true);
        l.setText("Chosed Method");
        l.setLocation(new Point(450, 10));
        return l;
    }

    private JLabel getDuplicateMethodsLabel() {
        JLabel jl = new JLabel("Duplicate Methods") ;
        jl.setForeground(new Color(0, 255, 0));
        return jl;

    }

    private void initListPanel() {
        //getListPanel().setVisible(true);
        listPanel.setLayout(new GridLayout(1, 2));
    }



    private void initDoScrollPane() {
        selectedMethodsScrollPane.setSize(new Dimension(300, 190));
        selectedMethodsScrollPane.setVisible(true);
        selectedMethodsScrollPane.setLocation(new Point(370, 30));
        selectedMethodsScrollPane.getViewport().add(doJList);
    }

    private void buildScrollPane() {
        duplicateScrollPane.setSize(new Dimension(300, 190));
        duplicateScrollPane.setLocation(new Point(10, 30));
        duplicateScrollPane.getViewport().add(duplicateJList);
    }

    private boolean isShowing = false;

    public void addNotify() {
        jf.addNotify();
        if (isShowing) return;

        // move components to account for insets
        Insets insets = jf.getInsets();
        Component[] components = jf.getComponents();
        for (int i = 0;
             i < components.length;
             i++) {
            Point location = components[i].getLocation();
            location.move(location.x,
                    location.y +
                    insets.top);
            components[i].setLocation(location);
        }
        isShowing = true;
    }

    void exit() {
        int choice =
                JOptionPane.showConfirmDialog(null,
                        "Choose yes to exit, No to continue",
                        "Exit this application ",
                        JOptionPane.YES_NO_OPTION,
                        JOptionPane.QUESTION_MESSAGE);
        if (choice == 0) {
            jf.setVisible(false);
            jf.dispose();
            System.exit(0);
        }
    }

    public void doListValueChanged(javax.swing.event.ListSelectionEvent e) {
    }

    private DelegateSynthesizer getDelegateSynthesizer() {
        if (ds == null)
            ds = new DelegateSynthesizer();
        return ds;
    }

    private ReflectUtil getReflectUtil() {
        return reflectUtil;
    }

    private class AddListener
            implements
            java.awt.event.ActionListener {
        public void actionPerformed(java.awt.event.ActionEvent e) {
            doAdd();
        }
    }

    private void doAdd() {
        Object o =
                duplicateJList.getSelectedValue();
        int i = duplicateJList.getSelectedIndex();
        if (getDelegateSynthesizer()
                .getDupMethodList()
                .size() ==
                0 ||
                i == -1) {
            JOptionPane.showMessageDialog(null,
                    "Please select a method"
                    +
                    " from Duplicate Methods window",
                    "STOP",
                    JOptionPane.ERROR_MESSAGE);
            System.out.println("No selection");
            return;
        }
        System.out.println("Add " + o);
        getDelegateSynthesizer()
                .removeDupMethod(o);
        // add to do list
        getDelegateSynthesizer().addDoMethod(o);
        //re-display the duplicate list
        duplicateJList.setListData(getDelegateSynthesizer()
                .getDupMethodList());
        //refresh the do list
        doJList.setListData(getDelegateSynthesizer()
                .getDoMethodList());
        dupLabel.setText("Duplicate Methods: " +
                getDelegateSynthesizer()
                .getDupMethodList()
                .size());
        doLabel.setText("selected Method: " +
                getDelegateSynthesizer()
                .getDoMethodList()
                .size());
    }

    private class DelListener
            implements java.awt.event.ActionListener {
        public void actionPerformed(java.awt.event.ActionEvent e) {
            doDelete();
        }
    }

    private void doDelete() {
        Object o = doJList.getSelectedValue();
        int i = doJList.getSelectedIndex();
        if (getDelegateSynthesizer()
                .getDoMethodList()
                .size() ==
                0 ||
                i == -1) {
            JOptionPane.showMessageDialog(null,
                    "Please select a method from Chosed Methods Window",
                    "STOP",
                    JOptionPane.ERROR_MESSAGE);
            System.out.println("No selection");
            return;
        }
        System.out.println("Del " + o);
        //delete from the do list
        getDelegateSynthesizer()
                .removeDoMethod(o);
        // add to dup list
        getDelegateSynthesizer()
                .addDupMethod(o);
        //re-display the duplicate list
        duplicateJList.setListData(getDelegateSynthesizer()
                .getDupMethodList());
        //refresh the do list
        doJList.setListData(getDelegateSynthesizer()
                .getDoMethodList());
        dupLabel.setText("Duplicate Methods: " +
                getDelegateSynthesizer()
                .getDupMethodList()
                .size());
        doLabel.setText("selected Method: " +
                getDelegateSynthesizer()
                .getDoMethodList()
                .size());
    }

    private void doOk() {
        System.out.println("creating user chosen sorting file");
        DelegateSynthesizer ds = getDelegateSynthesizer();
        ds.chooseToplogic(false);
        ds.createFile();
        System.out.println(ds.getClassString() +
                ds.getInterface());
    }

    public void generateTopologicFile() {
        System.out.println("creating topologic sorting file");
        DelegateSynthesizer ds =
                getDelegateSynthesizer();
        ds.chooseToplogic(true);
        ds.createFile();
        System.out.println(ds.getClassString()
                + ds.getInterface());
    }
}